/////////////////////////////////////////////////////////////////////////////////////////////+
//===========================================================================================+
//+------------------------------------------------------------------------------------------+
//|                                                                           [ea]vgFANN.mq4 |
//|                                               Copyright  2009, VladVG, VladislavVG (VG) |
//|                                                                           4vg AT mail.ru |
//+------------------------------------------------------------------------------------------+
//===========================================================================================+
/////////////////////////////////////////////////////////////////////////////////////////////+
//===========================================================================================+
//|                                   FANN-EA                                       |
//|                              Mariusz Woloszyn & Yury V. Reshetov                         |
//|                                           Fann2MQL.wordpress.com                         |
//|                                       Modify by Yury V. Reshetov                         |
//===========================================================================================+
/////////////////////////////////////////////////////////////////////////////////////////////+

#property copyright "Copyright  2009, Mariusz Woloszyn & Yury V. Reshetov, VladVG, VladislavVG (VG)"
#property link      "4vg AT mail.ru"

// Include Neural Network package
#include <vgTALib.mqh>
#include <hTradeServFuncs.mqh>
#include <hStrategyEnterPointFinder.mqh>

// Global defines
#define ANN_DISCK ""
#define ANN_PATH	"experts\\files\\ANN"
#define FANN_EXT  "fann"
string  ANN_FNAME ="";
// EA Name
#define NAME		"[ea]vgFANN"
#define MGKNUMB   110101
//---- input parameters
extern double StopLoss  = 81.0;
extern double TakePrfit = 81.0;

extern int    x    =   0;
extern double Lots = 0.1;

static int prevtime = 0;

#define MAX_ANNS_COUNT   21
#define MAX_ANNS_LAYERS   5
#define MAX_ANNS_INPUT  500
#define MAX_ANNS_OUTPUT   3
double  ANN_IN[ MAX_ANNS_COUNT][ MAX_ANNS_INPUT];
double  ANN_OUT[MAX_ANNS_COUNT][MAX_ANNS_OUTPUT];
//int     CommityTopology[MAX_ANNS_COUNT][MAX_ANNS_LAYERS];


#define ANNS_COUNT 16

static int    AnnsNumber  =  ANNS_COUNT;
static int    AnnInputs   = 30;
static bool   NeuroFilter = true;
static int    DebugLevel  = 2;
static double MinimalBalance = 200;
static bool   Parallel = false;
static bool   SaveAnn  =  true;
#define FANN_INPUTsCOUNT     30
#define FANN_HdLvl1COUNT     30
#define FANN_HdLvl2COUNT     16
#define FANN_OUTPUTsCOUNT     1

// ANN Structure
int FANN_LAYERS_COUNT     = 4;
int FANN_NEURONsInLayer[] = {FANN_INPUTsCOUNT,FANN_HdLvl1COUNT,FANN_HdLvl2COUNT,FANN_OUTPUTsCOUNT};
int initFANN()
{
int LrsCnt = 0;
    return(LrsCnt);
}

// Global variables

// Path to anns folder
static string AnnPath = "";

// Trade magic number
static int id = 0;

// AnnsArray[ann#] - Array of anns
static int    AnnsArray[MAX_ANNS_COUNT];

// All anns loded properly status
static bool   AnnsLoaded = true;

// AnnOutputs[ann#] - Array of ann returned returned
static double OutputVector[MAX_ANNS_INPUT];
static int    AnnOutputsSz[MAX_ANNS_INPUT];

// InputVector[] - Array of ann input data
static double InputVector[MAX_ANNS_INPUT];
//static double InputVector1[];

// Remembered long and short network inputs

/* Load the ANN */
int ann_load(string path, bool PrnResult=true, bool SaveBeginnerWeight=true) 
{
int ann = -1;

    if(path!="")
        ann = fann4MT_create_from_file (path);
    
    if (ann == -1) 
    {
	     /* Create ANN */
	     if(FANN_NEURONsInLayer[0]>=MAX_ANNS_INPUT)
	     {
	         Print ( "ERROR INPUT SIZE ! " + path);
            return (-1);
	     }
	     if(FANN_NEURONsInLayer[FANN_LAYERS_COUNT-1]>=MAX_ANNS_OUTPUT)
	     {
	         Print ( "ERROR OUTPUT SIZE ! " + path);
            return (-1);
	     }
	     ann = fann4MT_create_standard (FANN_LAYERS_COUNT, FANN_NEURONsInLayer);
        if (ann == -1) 
        {
	         Print ( "ERROR INITIALIZING NETWORK! " + path);
            return (ann);
        }
	     fann4MT_set_act_function_hidden (ann, FANN_SIGMOID_SYMMETRIC_STEPWISE);
	     fann4MT_set_act_function_output (ann, FANN_SIGMOID_SYMMETRIC_STEPWISE);
	     fann4MT_randomize_weights (ann, -1.0, 1.0);
	     if(SaveBeginnerWeight)
	     {
	         string full_path = GetFullFileName(ANN_DISCK, ANN_PATH, "bgSave_"+ANN_FNAME, FANN_EXT);
	         ann_save(ann, full_path, true );
	     }
    }
    if(PrnResult) Print("ANN: '" + path + "' created successfully with handler " + ann);
    return (ann);
}//int ann_load (string path) 


/* Save the ANN */
int ann_save(int ann, string path, bool PrnResult=true ) 
{
int ret = fann4MT_save (ann, path);
    if(PrnResult) Print("fann4MT_save(" + ann + ", " + path + ") returned: " + ret);
    return(ret);
}

int SaveANNs(bool NeedDestroy=false)
{
int k=0;

    for (int i = AnnsNumber-1; i >=0 ; i--,k++) 
    {
        //string full_path = GetFullFileName(ANN_DISCK, ANN_PATH, preficks+"_"+ANN_FNAME+"."+i, FANN_EXT);
	     ann_save (AnnsArray[i], AnnPath + "." + i + "." + FANN_EXT);
	     if(NeedDestroy) ann_destroy (AnnsArray[i]);
    }
    return(k);
}

void ann_destroy (int ann, bool PrnResult=true ) 
{
int ret = fann4MT_destroy (ann);
    if(PrnResult) Print("fann4MT_destroy(" + ann + ") returned: " + ret);
    return(ret);
}

int ann_run (int ann, double input[], double &output[], bool PrnResult=true ) 
{
int ret = fann4MT_run (ann, input);
//double out;
    if (ret < 0) 
    {
	     Print("Network RUN ERROR! ann= " + ann);
	     return (FANN_DOUBLE_ERROR);
    }
int outputSz = fann4MT_get_output_vector(ann, output);
    //out = fann4MT_get_output(ann, 0);
    if(outputSz<0)
    {
	     Print("Network RUN ERROR! OUTPUT Vector ann(" + ann +") Size = " + outputSz);
	     return (FANN_DOUBLE_ERROR);
    }
    if(PrnResult) Print("fann4MT_get_output_vector(" + ann + ") returned: " + outputSz);
    return(outputSz);
}

int RSI_Periods[] = {5,5,8,8,13,13,21,21,34,34,55,55,89,89,144,144};
void ann_prepare_inputs4Commity()
{
int i=0;
double res = 0;
    for(int iann=0;iann<AnnsNumber;iann++)
    {
         for(i = 0; i < AnnInputs; i++) 
	      {
             res = (iRSI(Symbol(), 0, RSI_Periods[iann], PRICE_OPEN, i) - 50.0) / 50.0; 
             if (MathAbs(res) > 1) 
             {
                 if (res > 0) 
                 {
                     ANN_IN[iann][i] = 1.0;            
                 } 
                 else 
                 {
                     ANN_IN[iann][i] = -1.0;            
                 }
             } 
             else 
             {
                 ANN_IN[iann][i] = res;            
             }
         }
    }
}//void ann_prepare_inputCommity()

void ann_set_input(double& in[], int iann)
{
    for(int i=0;i<AnnInputs; i++)
        in[i] = ANN_IN[iann][i];
}

void ann_set_output(double out[], int iann)
{
    for(int i=0;i<AnnOutputsSz[iann]; i++)
        ANN_OUT[iann][i] = out[i];
}


void ann_prepare_input () 
{
int i=0;
double res = 0;
    for(i = 0; i < AnnInputs; i++) 
	 {
        res = (iRSI(Symbol(), 0, 30, PRICE_OPEN, i) - 50.0) / 50.0; 
        if (MathAbs(res) > 1) 
        {
            if (res > 0) 
            {
                InputVector[i] = 1.0;            
            } 
            else 
            {
                InputVector[i] = -1.0;            
            }
        } 
        else 
        {
            InputVector[i] = res;            
        }
    }
}

// Get Outpust

void run_anns () 
{
int i;
    //ann_prepare_inputs4Commity();
    for (i = 0; i < AnnsNumber; i++) 
    {
        ann_set_input(InputVector, i);
	     AnnOutputsSz[i] = ann_run(AnnsArray[i], InputVector, OutputVector);
	     ann_set_output(OutputVector, i);
	 }
}//void run_anns ()

void ann_train(int ann, double &input_vector[], double &output_vector[], bool PrnResult=true ) 
{
    if (fann4MT_train (ann, input_vector, output_vector) == -1) 
    {
	     Print("Network TRAIN ERROR! ann=" + ann);
    }
    if(PrnResult) Print("ann_train(" + ann + ") succeded");
}

// PNN section

double ann_pnn() 
{
double ret=0;

    if (AnnsNumber < 1) 
    {
	     return (-1);
	 }

    for (int i = 0; i < AnnsNumber; i++) 
    {
        for(int j=0;j<AnnOutputsSz[i];j++)
	         ret += ANN_OUT[i][j];//AnnOutputs[i];
    }

    ret = ret / AnnsNumber;

    Print("Wise result: " + ret);
    return (ret);
}

//GetFullFileName(string DISK, string PATH, string FName, string FEXT);

//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init () {
    int i, ann;

    prevtime = Time[0];

    if (AnnInputs < 3) 
    {
	     Print("AnnInputs (",AnnInputs,")<3  too low!");
    }
    // Compute AnnPath
    id = StopLoss;
    ANN_FNAME = StringConcatenate( Symbol(), "-", StopLoss, "-", TakePrfit);
    AnnPath   = StringConcatenate( ANN_PATH,"\\", ANN_FNAME);

    // Initialize anns
    //ArrayResize (AnnsArray, AnnsNumber);
    for (i = 0; i < AnnsNumber; i++) 
    {
	     ann = ann_load(AnnPath + "." + i + "." + FANN_EXT);
	     if (ann < 0) 
	     {
	        AnnsLoaded = false;
	     }
	     AnnsArray[i] = ann;
    }
    //ArrayResize (AnnOutputs, AnnsNumber);
    //ArrayResize (InputVector, AnnInputs);

    return (0);
}


//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit () 
{
    int i;

    SaveANNs(true);
    // Deinitialize Intel TBB threads
    //fann4MT_parallel_deinit ();

    return (0);
}

bool trade_allowed () 
{
    if (!AnnsLoaded) 
    {
	     return (false);
	 }

    /* Trade only if there's enough funds */
    if(AccountBalance () > MinimalBalance) 
    {
	     return (true);
    }

    return (false);
}

//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start() 
{

    if(prevtime == Time[0]) 
    {
      return(0);
    }
    prevtime = Time[0];
    
    //int i = 0;

    double train_output[MAX_ANNS_OUTPUT];
    


    /* Is trade allowed? */
    if (!trade_allowed ()) 
    {
	     return (-1);
    }


   //int total = OrdersTotal();
   for (int i = OrdersTotal()-1; i >=0; i--) 
   {
      if(!OrderSelect(i, SELECT_BY_POS, MODE_TRADES)) break;
      if(OrderSymbol() == Symbol()) 
      {
         return(0);// Alwase in Trade. If order Exist - We are waiting Stop or Profit
      }
   }

   /* Prepare and run neural networks */
   ann_prepare_inputs4Commity();
   //ann_prepare_input ();
   
   // ************ Adaptive part ************
    if (IsOptimization() || IsTesting()) 
    {
        int total = OrdersHistoryTotal();
        if (total > 0) 
        {
            OrderSelect(total - 1, SELECT_BY_POS, MODE_HISTORY);   
            if (OrderProfit() < 0) 
            {
                if (OrderType() == OP_SELL) 
                {
                    train_output[0] = 1; 
                } 
                else 
                {
                    train_output[0] = -1; 
                }
                // Learning
		          for(int j=0;j<AnnsNumber;j++)
		          {
		              ann_set_input(InputVector, j);
		              ann_train (AnnsArray[j], InputVector, train_output);
		          }
            }//if (OrderProfit() < 0) 
        }//if(total > 0) 
    }//if (IsOptimization() || IsTesting()) 
    // ************ End of Adaptive part ************
   
   
   // Get Outputs
   run_anns();
   // Get Results
   double res = ann_pnn();
   
   // Trade
   
   int ticket = 0;
   
   RefreshRates();
   if (res > 0) 
   {
      if(IsTesting()||IsOptimization())
          ticket = OrderSend(Symbol(), OP_BUY, Lots, Ask, 2, Ask -  StopLoss * Point, Ask + TakePrfit * Point, WindowExpertName(), 0, 0, Blue);
      else 
          ticket = SetMarketOrder( nSYMBOL, OP_BUY, Lots, Ask, 2, Ask-StopLoss*Point, Ask+TakePrfit*Point, WindowExpertName(), MGKNUMB, 0, Blue );
      
   } 
   else if(res<0)
   {
      if(IsTesting()||IsOptimization())
          ticket = OrderSend(Symbol(), OP_SELL, Lots, Bid, 2, Bid +  StopLoss * Point, Bid - TakePrfit * Point, WindowExpertName(), 0, 0, Red);
      else 
          ticket = SetMarketOrder( nSYMBOL, OP_SELL, Lots, Bid, 2, Bid+StopLoss*Point, Ask-TakePrfit*Point, WindowExpertName(), MGKNUMB, 0, Red );
      //OrderSend(Symbol(), OP_SELL, Lots, Bid, 2, Bid +  StopLoss * Point, Bid - StopLoss * Point, WindowExpertName(), 0, 0, Red);
   }
   /*
   if (ticket >= 0) 
   {
      //ann_prepare_input ();
      ann_prepare_inputs4Commity();
   }
   */
   return (0);
}